rm(list=ls())
library(snowfall)
library(rlecuyer)


parallel_func=function(arg){
library(quantreg)


########################## data_generate function begin

data_generate=function(T,beta0,betax, betay,rho, phi,mu0,sd0,tau0){
m=100;
e_x=rnorm(T+2+m)*0.5
e=(rlnorm(T+2+m,sd=mu0)-qlnorm(tau0,sd=mu0))/sd0

x=rep(0,T+2+m)
e_y=rep(0,T+2+m)
for(t in 2:(T+2+m)){
            x[t]=0.0004+rho*x[t-1]+e_x[t]
            e_y[t]=phi*e_y[t-1]+e[t]
                 }


y=rep(0,T+2+m)

for(t in 3:(T+2+m)){
            y[t]=beta0+betax*x[t-1]+ betay*y[t-1] +e_y[t]
                 }

return(cbind(x[(m+2):(T+1+m)],y[(m+2):(T+1+m)],y[(m+3):(T+2+m)]))

                                        }


########################## data_generate function end






########################## beta_est function begin  
beta_est=function(tau0, XX,y){
 
model <- rq( y~XX, tau = tau0,ci=FALSE)
m=length(y)
res=residuals(model)
res1=(tau0-(res<0))
res2=(tau0-(res<0))*XX[,2]
K=1
mu=mean(res[-c(1:K)])
#mu=mean(res)
cor1=mean((res1[-c(1:K)]-mu*0)*((res1[-c(m-1:K)]-mu*0)))
 
return(list(coeff=model$coefficients[2:3],pv=summary(model,se="boot")$coefficients[2:3,4],cor=cor1))

                           }

########################## beta_est function end


########################## beta_est_rwb function begin

beta_est_rwb=function(tau0,XX,y,delta){
 
model_boot <- rq( y~XX, tau = tau0,ci=FALSE,weights=delta)
res_boot=residuals(model_boot)
m=length(y)
res1_boot=(tau0-(res_boot<0))
res2_boot=(tau0-(res_boot<0))*XX[,2]
K=1
mu_boot=sum(delta[-c(1:K)]*res_boot[-c(1:K)])/sum(delta[-c(1:K)])
#mu_boot=sum(delta*res_boot)/sum(delta)
cor1=sum(delta[-c(1:K)]*(res1_boot[-c(1:K)]-mu_boot*0)*((res1_boot[-c(m-1:K)]-mu_boot*0)))/sum(delta[-c(1:K)])
 


return(c(model_boot$coefficients[2:3],cor1))

                             }
########################## beta_est_rwb function end




########################## rwb_pvalues function begin  

rwb_pvalues=function(tau0, betas_hats,cor1_hat, XX,y, T, B){  ###calculate the rwb  p values

betas_rwb=matrix(rep(0,B*3),B, 3)

for (k in 1:B){    ###k begin
   delta_k=rexp(T,1)
   betas_rwb[k,]=beta_est_rwb(tau0,XX,y,delta_k) 
                }    ###k end  



betax_pvalue=mean(abs(betas_hats[1])<abs(betas_rwb[,1]-betas_hats[1]))
betay_pvalue=mean(abs(betas_hats[2])<abs(betas_rwb[,2]-betas_hats[2]))
cor1_pvalue1=mean(abs(cor1_hat)<abs((betas_rwb[,3]-cor1_hat)))
 

return(cbind(betax_pvalue, betay_pvalue,cor1_pvalue1))

                   }                                

####################  rwb_pvalues function end

##########power function for model 1

power_model1=function(T,tau0,beta0,betax, betay,rho, phi, B,rep,mu0,sd0){

                     
                 pvalues=matrix(rep(0,rep*5), rep,5)
                 for(j in 1:rep){
                 data=data_generate(T,beta0,betax, betay,rho, phi,mu0,sd0,tau0)
                 XX=data[, 1:2]
                 y=data[,3]
                 temp=beta_est(tau0,XX,y)
                 betas_hats=temp$coeff
                 pvalues[j,1:2]=temp$pv
cor1_hat=temp$cor
                 pvalues[j,3:5]=rwb_pvalues(tau0, betas_hats,cor1_hat, XX,y, T, B)
                  print(c(j,pvalues[j,]))
                                }

                #indicater=matrix(as.integer(as.logical(pvalues<0.05)), rep,6)
               
               return(pvalues)



 } 





#########we set rho=0.86 since the correlation coefficient for NFCI is 0.86
##########power results, the  column 1 and 2 is the classical quantile P values
#########the  column 3 and 4 is the rwb1  P values for the coeff of x_{t-1} and y_{t-1}
#########the  column 5 and 6 is the rwb2  P values for  the coeff of x_{t-1} and y_{t-1}
############model1: yt~x_{t-1}+y_{t-1}
###size


#Case=1;r=1;T=2000;mu0=c(0.5,1,1.5);sd0=c(0.2053,0.7313,3.2477);tau=0.1;phi0=0.;beta0=-0.9551;betay=0.1762;betax=-2.0146;

#Case=2;r=1;T=200;mu0=c(0.5,1,1.5);sd0=c(0.2072,0.7668,3.2222);tau=0.5;phi0=0;beta0=2.2420;betay=0.1297;betax=-1.0056; 


##power

Case=3;r=1;T=2000;mu0=c(0.5,1,1.5);sd0=c(0.2053,0.7313,3.2477);tau=0.1;phi0=0.5;beta0=-0.9551;betay=0.1762;betax=-2.0146;

#Case=4;r=1;T=200;mu0=c(0.5,1,1.5);sd0=c(0.2072,0.7668,3.2222);tau=0.5;phi0=0.5;beta0=2.2420;betay=0.1297;betax=-1.0056; 





 
res=power_model1(T=T,tau0=tau,beta0=beta0,betax=betax,betay=betay, rho=0.86, phi=phi0, B=1000,rep=200,mu0[r],sd0[r])
print(apply(res<0.1,2,mean))
save(res,file=paste0('Simu_Case',Case,'_n',T,'_r',r,'_', arg,'.Rdata'))
}
#job_id=as.numeric(Sys.getenv('SLURM_ARRAY_TASK_ID'))
#sfInit(parallel = TRUE, cpus =1, type = 'SOCK')
#sfClusterSetupRNGstream(seed = rep(job_id,4))
#res <- sfLapply(job_id, parallel_func)
#sfStop()
job_id=1;set.seed(rep(job_id,4));parallel_func(job_id)

